@@ -10,15 +10,20 @@ class DCCA(_DCCA_base):
     """
     A class used to fit a DCCA model.
 
-    Examples
-    --------
-    >>> from cca_zoo.deepmodels import DCCA
-    >>> model = DCCA()
+    :Citation:
+
+    Andrew, Galen, et al. "Deep canonical correlation analysis." International conference on machine learning. PMLR, 2013.
+
     """
 
-    def __init__(self, latent_dims: int, objective=objectives.MCCA,
-                 encoders=None,
-                 r: float = 0, eps: float = 1e-3):
+    def __init__(
+        self,
+        latent_dims: int,
+        objective=objectives.MCCA,
+        encoders=None,
+        r: float = 0,
+        eps: float = 1e-5,
+    ):
         """
         Constructor class for DCCA
 
@@ -35,23 +40,25 @@ class DCCA(_DCCA_base):
         self.objective = objective(latent_dims, r=r, eps=eps)
 
     def forward(self, *args):
-        z = self.encode(*args)
-        return z
-
-    def encode(self, *args):
         z = []
         for i, encoder in enumerate(self.encoders):
             z.append(encoder(args[i]))
-        return tuple(z)
+        return z
 
     def loss(self, *args):
+        """
+        Define the loss function for the model. This is used by the DeepWrapper class
+
+        :param args:
+        :return:
+        """
         z = self(*args)
         return self.objective.loss(*z)
 
-    def post_transform(self, *z_list, train=False):
+    def post_transform(self, z_list, train=False):
         if train:
             self.cca = MCCA(latent_dims=self.latent_dims)
-            z_list = self.cca.fit_transform(*z_list)
+            z_list = self.cca.fit_transform(z_list)
         else:
-            z_list = self.cca.transform(*z_list)
+            z_list = self.cca.transform(z_list)
         return z_list
