@@ -9,14 +9,13 @@ class SplitAE(_DCCA_base):
     """
     A class used to fit a Split Autoencoder model.
 
-    Examples
-    --------
-    >>> from cca_zoo.deepmodels import SplitAE
-    >>> model = SplitAE()
+    :Citation:
+
+    Ngiam, Jiquan, et al. "Multimodal deep learning." ICML. 2011.
+
     """
 
-    def __init__(self, latent_dims: int, encoder: BaseEncoder = Encoder,
-                 decoders=None):
+    def __init__(self, latent_dims: int, encoder: BaseEncoder = Encoder, decoders=None):
         """
 
         :param latent_dims: # latent dimensions
@@ -30,26 +29,37 @@ class SplitAE(_DCCA_base):
         self.decoders = torch.nn.ModuleList(decoders)
 
     def forward(self, *args):
-        z = self.encode(*args)
-        return z
-
-    def encode(self, *args):
         z = self.encoder(args[0])
-        return z
+        return [z]
 
-    def decode(self, z):
+    def recon(self, *args):
+        """
+        :param args:
+        :return:
+        """
+        z = self(*args)
+        return self._decode(z)
+
+    def _decode(self, *z):
+        """
+        This method is used to decode from the latent space to the best prediction of the original views
+
+        :param z:
+        """
         recon = []
         for i, decoder in enumerate(self.decoders):
-            recon.append(decoder(z))
+            recon.append(decoder(*z))
         return tuple(recon)
 
     def loss(self, *args):
-        z = self.encode(*args)
-        recon = self.decode(z)
+        z = self(*args)
+        recon = self._decode(*z)
         recon_loss = self.recon_loss(args, recon)
         return recon_loss
 
     @staticmethod
     def recon_loss(x, recon):
-        recons = [F.mse_loss(recon[i], x[i], reduction='mean') for i in range(len(x))]
+        recons = [
+            F.mse_loss(recon[i], x[i], reduction="mean") for i in range(len(recon))
+        ]
         return torch.stack(recons).sum(dim=0)
